@@ -9,8 +9,10 @@ import valid_experiments_time
 
 import copy
 
+import remote_archive
+
 #import database_utils
-import json_tools
+import io_tools
 
 import tree_utils
 from tree_utils import File_Expt
@@ -18,7 +20,17 @@ from tree_utils import File_Expt
 import cdb_query_archive_parsers
 import cdb_query_archive_class
 
-def main():
+import netcdf_utils
+
+def main_CMIP5():
+    main('CMIP5')
+    return
+
+def main_CORDEX():
+    main('CORDEX')
+    return
+
+def main(project):
     import argparse 
     import textwrap
 
@@ -39,26 +51,34 @@ def main():
                             version='%(prog)s '+version_num,
                             epilog=epilog)
 
-    subparsers = parser.add_subparsers(help='commands',dest='command')
+    project_drs=remote_archive.DRS(project)
 
     #Generate subparsers
-    cdb_query_archive_parsers.generate_subparsers(subparsers,epilog)
+    cdb_query_archive_parsers.generate_subparsers(parser,epilog,project_drs)
 
     options=parser.parse_args()
 
-    #Slicing time is perculiar
+    #Slicing time is peculiar
     for time_opt in ['year','month']:
         if time_opt in dir(options) and getattr(options,time_opt):
             options.time=True
 
     #Load pointer file:
-    paths_dict=cdb_query_archive_class.SimpleTree(json_tools.open_json(options))
-    #Run the command:
-    getattr(paths_dict,options.command)(options)
+    if options.command=='remote_retrieve':
+        netcdf_utils.retrieve_data(options)
+    elif 'in_diagnostic_headers_file' in dir(options):
+        paths_dict=cdb_query_archive_class.SimpleTree(io_tools.open_json(options),options,project_drs)
+        #Run the command:
+        getattr(paths_dict,options.command)(options)
+    elif 'in_diagnostic_netcdf_file' in dir(options):
+        paths_dict=cdb_query_archive_class.SimpleTree(io_tools.open_netcdf(options),options,project_drs)
+        #Run the command:
+        getattr(paths_dict,options.command)(options)
+        
     #print paths_dict.pointers.tree
     #Close the file:
     if 'out_diagnostic_headers_file' in dir(options):
-        json_tools.close_json(paths_dict,options)
+        io_tools.close_json(paths_dict,options)
 
 if __name__ == "__main__":
-    main()
+    main('CMIP5')
